@@ -14,6 +14,8 @@
 
 import math
 
+import pytest
+
 from slidingpuzzle import *
 
 
@@ -64,7 +66,7 @@ def test_last_moves_distance():
     board = board_from([1, 2, 3], [4, 5, 6], [7, 0, 8])
     assert last_moves_distance(board) == 0
 
-    board = board_from([8, 2, 3], [4, 5, 6], [7, 0, 1])
+    board = board_from([8, 6, 0], [4, 5, 2], [7, 3, 1])
     assert last_moves_distance(board) == 2
 
 
@@ -73,13 +75,13 @@ def test_linear_conflict_distance():
     assert linear_conflict_distance(board) == 1
 
     board = board_from([2, 1, 3], [4, 5, 6], [7, 8, 0])
-    assert linear_conflict_distance(board) == 6
+    assert linear_conflict_distance(board) == 4
 
     board = board_from([4, 2, 3], [1, 5, 6], [7, 8, 0])
-    assert linear_conflict_distance(board) == 6
+    assert linear_conflict_distance(board) == 4
 
     board = board_from([2, 1, 3], [4, 5, 6], [7, 0, 8])
-    assert linear_conflict_distance(board) == 7
+    assert linear_conflict_distance(board) == 5
 
     board = board_from([1, 2, 3], [6, 5, 4], [7, 8, 0])
     assert linear_conflict_distance(board) == 8
@@ -104,3 +106,41 @@ def test_relaxed_adjacency_distance():
     swap_tiles(b, (0, 0), (h - 1, w - 1))
     swap_tiles(b, (0, 0), (1, 0))
     assert relaxed_adjacency_distance(b) == 3
+
+
+@pytest.mark.slow
+def test_heuristic_behavior():
+    # we compute avg generated nodes over multiple runs to confirm that
+    # heuristic behavior is in line with expectations
+    generated_avg = compare(
+        3, 3, linear_conflict_distance, manhattan_distance, num_iters=4
+    )
+    assert generated_avg[0] < generated_avg[1]
+
+    generated_avg = compare(3, 3, manhattan_distance, hamming_distance, num_iters=4)
+    assert generated_avg[0] < generated_avg[1]
+
+    # lcd/manhattan/euclidean are good contenders, so we don't compare them
+    generated_avg = compare(3, 3, euclidean_distance, hamming_distance, num_iters=4)
+    assert generated_avg[0] < generated_avg[1]
+
+
+@pytest.mark.slow
+def test_heuristic_admissibility():
+    # validate that solutions are in line with BFS
+    # this does not guarantee admissibility, it's just an empirical sanity check
+    boards = [shuffle_board(new_board(3, 3)) for _ in range(50)]
+    optimal = [len(search(b, "bfs").solution) for b in boards]
+    for h in (linear_conflict_distance, manhattan_distance):
+        for b, o in zip(boards, optimal):
+            assert len(search(b, heuristic=h).solution) == o
+
+
+@pytest.mark.skip
+def test_linear_conflict_distance_exhaustive():
+    start = 0
+    stop = None
+    for i, b in enumerate(board_generator(3, 3, start, stop)):
+        expected = len(search(b, heuristic=manhattan_distance).solution)
+        actual = len(search(b, heuristic=linear_conflict_distance).solution)
+        assert expected == actual, f"{start + i}: {b}"
